/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.awt.Dimension;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.io.IOException;
import java.util.ArrayList;
import java.util.Collections;
import nl.cloudfarming.client.crop.model.Crop;
import java.util.List;
import javax.swing.Action;
import javax.swing.JComponent;
import javax.swing.JPanel;
import javax.swing.JToolBar;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.CropFactory;
import nl.cloudfarming.client.crop.model.CropManager;
import nl.cloudfarming.client.crop.model.CropService;
import nl.cloudfarming.client.crop.model.CropsStorage;
import nl.cloudfarming.client.crop.model.Genus;
import nl.cloudfarming.client.crop.model.GenusFactory;
import nl.cloudfarming.client.crop.model.Race;
import nl.cloudfarming.client.crop.model.RaceFactory;
import nl.cloudfarming.client.crop.type.CropsStorageDataObject;
import org.netbeans.api.project.Project;
import org.netbeans.api.project.ProjectManager;
import org.netbeans.core.spi.multiview.CloseOperationState;
import org.netbeans.core.spi.multiview.MultiViewElement;
import org.netbeans.core.spi.multiview.MultiViewElementCallback;
import org.netbeans.core.spi.multiview.text.MultiViewEditorElement;
import org.openide.awt.UndoRedo;
import org.openide.filesystems.FileObject;
import org.openide.util.Exceptions;
import org.openide.util.Lookup;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;
import org.openide.util.lookup.ProxyLookup;
import org.openide.windows.TopComponent;

/**
 * Visual tab for crops. This panel enables users to create new crops and alter existing crops
 * @author Merijn Zengers
 */
@MultiViewElement.Registration(displayName = "#LBL_Crop_VISUAL",
iconBase = "nl/cloudfarming/client/stock/project/visual/box.png",
mimeType = "application/x-agrosense-stock",
persistenceType = TopComponent.PERSISTENCE_NEVER,
preferredID = "CropVisual",
position = 2000)
@Messages({"LBL_Crop_VISUAL=Crop list"})
public class CropVisualElement extends MultiViewEditorElement {

    private transient MultiViewElementCallback callback;
    private CropsPanel cropPanel;
    private JToolBar toolbar = new JToolBar();
    private CropsStorageDataObject cropStorageDataObject;
    private CropService<CropsStorageDataObject> cropService;
    private Project project;

    public CropVisualElement(Lookup lkp) {
        super(lkp);
        FileObject stock = lkp.lookup(FileObject.class);
        project = null;
        try {
            project = ProjectManager.getDefault().findProject(stock.getParent());
        } catch (IOException | IllegalArgumentException ex) {
            Exceptions.printStackTrace(ex);
        }

        cropService = Lookup.getDefault().lookup(CropService.class);
        try {
            cropStorageDataObject = cropService.getCropsStorage(project);
        } catch (IOException ex) {
            Exceptions.printStackTrace(ex);
        }

        cropPanel = new CropsPanel(cropStorageDataObject.getCropsStorage());
    }

    @Override
    public JComponent getVisualRepresentation() {
        return cropPanel;
    }

    @Override
    public JComponent getToolbarRepresentation() {
        return toolbar;
    }

    @Override
    public Action[] getActions() {
        return new Action[0];
    }

    @Override
    public Lookup getLookup() {
        return new ProxyLookup(super.getLookup(), cropStorageDataObject.getLookup());
    }

    @Override
    public void componentOpened() {
    }

    /**
     * When a component is closed empty rows must be removed
     */
    @Override
    public void componentClosed() {
        if (removeEmptyRows(cropStorageDataObject.getCropsStorage())) {
            CropManager manager = Lookup.getDefault().lookup(CropManager.class);
            if (manager != null) {
                manager.save(cropStorageDataObject.getCropsStorage(), cropStorageDataObject.getPrimaryFile());
            }
        }
    }

    /**
     * Method which removes empty elements
     * Elements can be Crop Race or Genus
     * When a Crop name is empty and none of the names of the genera and Races are filled
     * the crop will be removed from cropsStorage
     * When the crop name is not filled filed but there is a genus with a name and a race without
     * the crop will be preserved but the race will be removed and vice versa
     * When a crop name is filled but there is a race and a genus both without a name
     * both the genus and the and the race are removed, the crop is preserved
     * @param cropsStorage the cropStorage to check
     * @return boolean if there is any change to cropsStorage
     */
    protected boolean removeEmptyRows(CropsStorage cropsStorage) {
        boolean changed = false;
        ArrayList<Crop> remove = new ArrayList<>();
        ArrayList<Genus> removedGenus = new ArrayList<>();
        ArrayList<Race> removedRace = new ArrayList<>();

        for (Crop crop : cropsStorage.getCrops()) {
            boolean genusRemove = true;
            for (Genus genus : crop.getGenera()) {
                if (genus.getName() != null && !genus.getName().equals("")) {
                    genusRemove = false;
                } else {
                    removedGenus.add(genus);
                    changed = true;
                }
            }
            crop.getGenera().removeAll(removedGenus);
            boolean raceRemove = true;
            for (Race race : crop.getRaces()) {
                if (race.getName() != null && !race.getName().equals("")) {
                    raceRemove = false;
                } else {
                    removedRace.add(race);
                    changed = true;
                }
            }
            crop.getRaces().removeAll(removedRace);


            if ((crop.getName() == null || crop.getName().equals(""))) {
                if (crop.getGenera().isEmpty() && crop.getRaces().isEmpty()) {
                    remove.add(crop);
                    changed = true;
                } else {
                    if (genusRemove && raceRemove) {
                        remove.add(crop);
                        changed = true;
                    }
                }
            }
        }
        cropsStorage.getCrops().removeAll(remove);
        return changed;
    }

    @Override
    public void componentShowing() {
    }

    @Override
    public void componentHidden() {
    }

    @Override
    public void componentActivated() {
    }

    @Override
    public void componentDeactivated() {
    }

    @Override
    public UndoRedo getUndoRedo() {
        return cropStorageDataObject.getUndoRedo();
    }

    @Override
    public void setMultiViewCallback(MultiViewElementCallback callback) {
        this.callback = callback;
    }

    @Override
    public CloseOperationState canCloseElement() {
        return CloseOperationState.STATE_OK;
    }

    @Messages({"CropPanelVisual.cropLabel=Name", "CropPanelVisual.cropsHeader=Crops", "CropPanelVisual.genusHeader=Class", "CropPanelVisual.genusLabel=Name", "CropPanelVisual.racesHeader=Races", "CropPanelVisual.raceLabel=Name", "cropPanelVisual.help.text=This panel enables you to add and remove crops and there respective classes and races. You browse through the crops by selecting one"})
    private class CropsPanel extends JPanel implements PropertyChangeListener {

        private JPanel racesGenusPanel;
        private ListController<Crop> cropsList;
        private ListController<Genus> genusList;
        private ListController<Race> racesList;
        private List<Crop> crops;
        private CropsStorage cropsStorage;
        private final CropFactory cropFactory = new CropFactory();
        private final GenusFactory genusFactory = new GenusFactory();
        private final RaceFactory raceFactory = new RaceFactory();
        private final String HELP_TEXT = NbBundle.getMessage(CropsPanel.class, "cropPanelVisual.help.text");
        private final String CROP_HEADER_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.cropsHeader");
        private final String CROP_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.cropLabel");
        private final String GENUS_HEADER_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.genusHeader");
        private final String GENUS_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.genusLabel");
        private final String RACES_HEADER_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.racesHeader");
        private final String RACES_LABEL = NbBundle.getMessage(CropsPanel.class, "CropPanelVisual.raceLabel");
        private static final int LIST_WIDTH = 350;
        private static final int LIST_HEIGHT = 550;

        public CropsPanel(CropsStorage cropsStorage) {
            this.cropsStorage = cropsStorage;
            this.crops = cropsStorage.getCrops();
            init();
        }

        private void init() {
            this.setLayout(new MigLayout("wrap 2", "center, max", "center, max"));
            racesGenusPanel = new JPanel(new MigLayout("wrap 1, insets 0 0 0 0"));
            racesGenusPanel.setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT));

            cropsList = new ListController(cropFactory, crops, CROP_HEADER_LABEL, CROP_LABEL);
            genusList = new ListController(genusFactory, crops.get(0).getGenera(), GENUS_HEADER_LABEL, GENUS_LABEL);
            racesList = new ListController(raceFactory, crops.get(0).getRaces(), RACES_HEADER_LABEL, RACES_LABEL);

            cropsList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT));
            genusList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT / 2));
            racesList.getComponent().setPreferredSize(new Dimension(LIST_WIDTH, LIST_HEIGHT / 2));


            cropsList.addPropertyChangeListener(ListController.PROP_SELECTION, this);

            cropsList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);
            genusList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);
            racesList.addPropertyChangeListener(ListController.PROP_ADD_ROW, this);

            cropsList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);
            genusList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);
            racesList.addPropertyChangeListener(ListController.PROP_REMOVE_ROW, this);

            cropsStorage.addPropertyChangeListener(CropsStorage.PROP_CROP, this);

            for (Crop crop : cropsStorage.getCrops()) {
                crop.addPropertyChangeListener(Crop.PROP_GENUS, this);
                crop.addPropertyChangeListener(Crop.PROP_RACE, this);
                crop.addPropertyChangeListener(Crop.PROP_NAME, this);
                for (Genus genus : crop.getGenera()) {
                    genus.addPropertyChangeListener(Genus.PROP_NAME, this);
                }
                for (Race race : crop.getRaces()) {
                    race.addPropertyChangeListener(Race.PROP_NAME, this);
                }
            }


            if (!crops.isEmpty()) {
                cropsList.setSelectedBean(crops.get(0));
                racesList.setBeans(crops.get(0).getRaces());
                genusList.setBeans(crops.get(0).getGenera());
            }

            this.add(cropsList.getComponent());
            racesGenusPanel.add(racesList.getComponent());
            racesGenusPanel.add(genusList.getComponent());

            this.add(racesGenusPanel, "align left");

        }

        @Override
        public void propertyChange(PropertyChangeEvent pce) {
            if (pce.getPropertyName().equals(ListController.PROP_SELECTION)) {
                if (pce.getNewValue() != null) {
                    genusList.setBeans(((Crop) pce.getNewValue()).getGenera());
                    genusList.setEnabled(true);
                    racesList.setBeans(((Crop) pce.getNewValue()).getRaces());
                    racesList.setEnabled(true);
                } else {
                    genusList.setBeans(Collections.EMPTY_LIST);
                    genusList.setEnabled(false);
                    racesList.setBeans(Collections.EMPTY_LIST);
                    racesList.setEnabled(false);
                }
            } else if (pce.getPropertyName().equals(ListController.PROP_ADD_ROW)) {
                Crop selectedCrop = cropsList.getSelectedBean();
                if (pce.getNewValue() instanceof Crop) {
                    cropsStorage.addCrop((Crop) pce.getNewValue());
                    ((Crop) pce.getNewValue()).addPropertyChangeListener(Crop.PROP_GENUS, this);
                    ((Crop) pce.getNewValue()).addPropertyChangeListener(Crop.PROP_RACE, this);
                } else if (pce.getNewValue() instanceof Genus) {
                    if (selectedCrop != null) {
                        selectedCrop.addGenus((Genus) pce.getNewValue());
                        ((Genus) pce.getNewValue()).addPropertyChangeListener(Genus.PROP_NAME, this);
                    }
                } else if (pce.getNewValue() instanceof Race) {
                    if (selectedCrop != null) {
                        selectedCrop.addRace((Race) pce.getNewValue());
                        ((Race) pce.getNewValue()).addPropertyChangeListener(Race.PROP_NAME, this);
                    }
                }
            } else if (pce.getPropertyName().equals(ListController.PROP_REMOVE_ROW)) {
                Crop selectedCrop = cropsList.getSelectedBean();
                if (pce.getOldValue() instanceof Crop) {
                    cropsStorage.removeCrop((Crop) pce.getOldValue());
                    ((Crop) pce.getOldValue()).removePropertyChangeListener(this);
                } else if (pce.getOldValue() instanceof Genus) {
                    if (selectedCrop != null) {
                        selectedCrop.removeGenus((Genus) pce.getOldValue());
                        ((Genus) pce.getOldValue()).removePropertyChangeListener(this);
                    }
                } else if (pce.getOldValue() instanceof Race) {
                    if (selectedCrop != null) {
                        selectedCrop.removeRace((Race) pce.getOldValue());
                        ((Race) pce.getOldValue()).removePropertyChangeListener(this);
                    }
                }
            } //Section for undo redo
            else if (pce.getPropertyName().equals(CropsStorage.PROP_CROP)) {
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    cropsList.setBeans((List) pce.getNewValue());
                }
            } else if (pce.getPropertyName().equals(Crop.PROP_GENUS)) {
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    genusList.setBeans((List) pce.getNewValue());
                }
            } else if (pce.getPropertyName().equals(Crop.PROP_RACE)) {
                if (pce.getOldValue() instanceof List && pce.getNewValue() instanceof List) {
                    racesList.setBeans((List) pce.getNewValue());
                }
            }
        }
    }
}
