/**
 * Copyright (C) 2011 Agrosense <info@cloudfarming.nl>
 *
 * Licensed under the Eclipse Public License - v 1.0 (the "License"); you may
 * not use this file except in compliance with the License. You may obtain a
 * copy of the License at
 *
 * http://www.eclipse.org/legal/epl-v10.html
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS, WITHOUT
 * WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied. See the
 * License for the specific language governing permissions and limitations under
 * the License.
 */
package nl.cloudfarming.client.stock.project.visual;

import java.awt.Component;
import java.awt.Dimension;
import java.awt.event.ActionEvent;
import java.awt.event.ActionListener;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import java.beans.PropertyChangeSupport;
import java.io.IOException;
import java.util.List;
import java.util.logging.Logger;
import javax.imageio.ImageIO;
import javax.swing.JLabel;
import javax.swing.JPanel;
import javax.swing.JScrollPane;
import javax.swing.JSeparator;
import javax.swing.border.EmptyBorder;
import javax.swing.border.EtchedBorder;
import net.miginfocom.swing.MigLayout;
import nl.cloudfarming.client.crop.model.CropBean;
import nl.cloudfarming.client.util.swing.button.ImageButton;
import org.openide.util.NbBundle;
import org.openide.util.NbBundle.Messages;

/**
 *
 * Creates a panel with rows for each provided bean.
 * Will Display provided header label
 * Followed by a row of JLabel JTextField and a delete button.
 * On click of the delete button the row will be removed from the panel.
 * To listen to this changes register as PropertyChangeListener on property ListPanel.PROP_ROW
 * In the propertyChangeEvent the bean which is removed will be the old value
 * @author Merijn Zengers
 */
@Messages("CropPanelVisual.add.button=add")
public class ListPanel<T extends CropBean> extends JPanel implements ActionListener, PropertyChangeListener {

    private PropertyChangeSupport propertyChangeSupport;
    private static final int SEPARATOR_WIDTH = 300;
    private JScrollPane pane;
    private JLabel header;
    private JSeparator separator;
    private JLabel raceLabel;
    private List<T> beans;
    public static final String PROP_ROW = "row";
    public static final String PROP_SELECTION = "selection";
    private String headerLabel;
    private String fieldLabel;
    private JPanel rowPanel;
    private static final String ADD_ICON = "add.png";
    private static final String ADD_SELECTED_ICON = "add_sel.png";
    private static final String ADD_DISABLED_ICON = "add_dis.png";
    private final String ADD_DESCRIPTION = NbBundle.getMessage(ListPanel.class, "CropPanelVisual.add.button");
    private BufferedImage addImage;
    private BufferedImage addSelectedImage;
    private BufferedImage addDisabledImage;
    private ImageButton addButton;
    private static final Logger LOGGER = Logger.getLogger("nl.cloudfarming.client.stock.project.visual");    

    /**
     * This will construct a panel containing the following
     * A header filed with the value provided in @param headerLabel
     * For every bean in @param beans a row of:
     * A JLabel as provided in @param fieldLabel
     * A JTextField for filling the name of the bean
     * A delete button
     * On keyReleases on the JtextField the provided bean.setName() will be invoked with the text of the JTextField 
     * @param beans the beans to build rows for
     * @param headerLabel the Label for the header of the panel
     * @param fieldLabel the label for every field
     */
    ListPanel(List<T> beans, String headerLabel, String fieldLabel) {
        this.beans = beans;
        this.headerLabel = headerLabel;
        this.fieldLabel = fieldLabel;
        init();
    }

    private void init() {
        this.setLayout(new MigLayout("wrap 1"));
        this.setBorder(new EtchedBorder(EtchedBorder.LOWERED));

        rowPanel = new JPanel();
        rowPanel.setLayout(new MigLayout("wrap 1, insets 0 0", "max"));
        header = new JLabel(headerLabel);
        separator = new JSeparator(JSeparator.HORIZONTAL);
        separator.setPreferredSize(new Dimension(SEPARATOR_WIDTH, 0));

        raceLabel = new JLabel(fieldLabel);
        raceLabel.setPreferredSize(new Dimension(Row.FIELD_WIDTH, 0));

        
        this.add(header);
        this.add(separator);
        this.add(raceLabel, "gapleft 10");
        
        if (beans != null && !beans.isEmpty()) {
            for (T bean : beans) {
                addRow(bean);
            }
        }
        
        pane = new JScrollPane(rowPanel);
        pane.setBorder(new EmptyBorder(0, 0, 0, 0));
        pane.setHorizontalScrollBarPolicy(JScrollPane.HORIZONTAL_SCROLLBAR_NEVER);
        pane.setVerticalScrollBarPolicy(JScrollPane.VERTICAL_SCROLLBAR_AS_NEEDED);
        this.add(pane);

        try {
            addImage = ImageIO.read(getClass().getResource(ADD_ICON));
            addSelectedImage = ImageIO.read(getClass().getResource(ADD_SELECTED_ICON));
            addDisabledImage = ImageIO.read(getClass().getResource(ADD_DISABLED_ICON));
            addButton = new ImageButton(addImage, addSelectedImage, addDisabledImage, this.getBackground(), ADD_DESCRIPTION);
            addButton.addActionListener(this);
            addButton.setFocusable(true);
            this.add(addButton, "gapleft 10");

        } catch (IOException ex) {
            LOGGER.fine("Could not load image " + ADD_ICON);
        }
    }

    /**
     * Adds a row to this Panel
     * @param the bean to add the row for
     */
    protected void addRow(T bean) {
        Row row = new Row(bean);
        row.addActionListener(this);
        row.addPropertyChangeListener(Row.PROP_SELECTED, this);
        rowPanel.add(row, "span");
        this.revalidate();
    }
    
    /**
     * Gets the selected row from the all the rows this ListPanel contains
     * @return the selected row or null if there is no selected row
     */
    public Row getSeleRow(){
        for(Component row : rowPanel.getComponents()){
            if(row instanceof Row){
                if(((Row)row).isSelected()){
                    return (Row) row;
                }
            }
        }
        return null;
    }
    
    /**
     * Sets the selected row
     * @param row the row to select.
     */
    public void setSelectedRow(Row row){
        for(Component component : rowPanel.getComponents()){
            if(component instanceof Row){
                if(component != row && ((Row) component).isSelected()){
                    ((Row) component).setSelected(false);
                } else if(component == row){
                    ((Row) component).setSelected(true);
                }   
            }
        }
    }
    
    /**
     * Gets a row based on the bean the row contains
     * @param bean the bean to search the row for
     * @return the row which contains the supplied bean or null if not found
     */
    public Row getRow(T bean){
        for(Component component : rowPanel.getComponents()){
            if(((Row) component).getBean() == bean){
                return (Row) component;
            }
        }
        return null;
    }
    
    
    /**
     * Method to set new a new list of beans
     * When set the panel will be updated and all the rows will be cleared and the new rows will be added 
     * @param beans 
     */
    public void setBeans (List<T> beans){
        this.beans = beans;
        redrawRows();
    }
    
    /**
     * Redraws the rows based on the beans list
     */
    private void redrawRows(){
        rowPanel.removeAll();
        for(T bean : beans){
            addRow(bean);
        }
        this.revalidate();
    }

    /**
     * Gets the button which is used to add rows 
     * @return the button
     */
    public ImageButton getAddButton() {
        return addButton;
    }

    @Override
    public void actionPerformed(ActionEvent ae) {
        if (ae.getSource() instanceof Row) {
            Row row = (Row) ae.getSource();
            rowPanel.remove((Component) ae.getSource());
            this.revalidate();
            getPropertyChangeSupport().firePropertyChange(PROP_ROW, row.getBean(), null);
        }
    }

    @Override
    public void addPropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(pcl);
    }

    @Override
    public void addPropertyChangeListener(String propertyName, PropertyChangeListener pcl) {
        getPropertyChangeSupport().addPropertyChangeListener(propertyName, pcl);
    }

    @Override
    public void removePropertyChangeListener(PropertyChangeListener pcl) {
        getPropertyChangeSupport().removePropertyChangeListener(pcl);
    }

    @Override
    public void removePropertyChangeListener(String propertyName, PropertyChangeListener pcl) {
        getPropertyChangeSupport().removePropertyChangeListener(propertyName, pcl);
    }

    PropertyChangeSupport getPropertyChangeSupport() {
        if (propertyChangeSupport == null) {
            propertyChangeSupport = new PropertyChangeSupport(this);
        }
        return propertyChangeSupport;
    }

    @Override
    public void propertyChange(PropertyChangeEvent pce) {
        if (pce.getPropertyName().equals(Row.PROP_SELECTED) && (Boolean) pce.getNewValue()) {
            Row old = null;
            for (Component component : rowPanel.getComponents()) {
                if (component instanceof Row) {
                    Row row = (Row) component;
                    if (row.isSelected() && pce.getSource() != row) {
                        row.setSelected(false);
                        old = row;
                    }
                }
            }
            getPropertyChangeSupport().firePropertyChange(PROP_SELECTION, old, pce.getSource());
        }
    }
}
